# Copyright 2017 Xiaomi, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# valueations under the License.

import re
import netaddr
from . import constant

kube_resource_name_regex = "[a-z0-9]([-a-z0-9]*[a-z0-9])?$"
kube_resource_name_regex_pattern = re.compile(kube_resource_name_regex)


def check_kube_resource_name_regex(resource_name):
  if kube_resource_name_regex_pattern.match(resource_name):
    return True
  else:
    return False


def check_cpu_value(value):
  if value != None:
    if not isinstance(value, str):
      raise ValueError("cpu value must be a string!")
    if not value.replace(".", "", 1).isdigit():
      raise ValueError("cpu value must be a number!")
    digits = value.split(".")
    if len(digits) == 2 and len(digits[1]) > constant.QUOTA_ACCURACY_PLACE:
      raise StandardError(
          "The value of cpu accurate to two decimal places, for example: {}".format(
              round(
                  float(value), constant.QUOTA_ACCURACY_PLACE)))
  return value


def check_memory_value(value):
  if value != None:
    if not isinstance(value, str):
      raise ValueError("memory value must be a string")
    unit = value[-1:]
    float_value = value[:-1]
    if unit not in constant.CLOUDML_MEMORY_UNITS:
      raise ValueError("memory value unit must be one of %s!" %
                       constant.CLOUDML_MEMORY_UNITS)
    if not float_value.replace(".", "", 1).isdigit():
      raise ValueError("memory value must be a number!")
    digits = float_value.split(".")
    if len(digits) == 2 and len(digits[1]) > constant.QUOTA_ACCURACY_PLACE:
      raise StandardError(
          "The value of memory accurate to two decimal places, for example: {}".format(
              round(
                  float(float_value), constant.QUOTA_ACCURACY_PLACE)))
  return value


def check_gpu_value(value):
  if value != None:
    if not (isinstance(value, str) and value.isdigit() and int(value) >= 0):
      raise ValueError("gpu value must be a string for nonnegative integer!")
    value = int(value)
  return value


def check_distributed_count(value):
  if value != None:
    if not (isinstance(value, str) and value.isdigit() and int(value) > 0):
      raise ValueError("Distributed task count must be a string for positive integer!")
    value = int(value)
  return value

def check_network_port(port):
  try:
    p = int(port)
    if p < constant.DEV_PROXY_MIN_CONTAINER_PORT or p > constant.DEV_PROXY_MAX_CONTAINER_PORT:
      raise ValueError("Network port should be in the range of [{}, {}]".format(constant.DEV_PROXY_MIN_CONTAINER_PORT,constant.DEV_PROXY_MAX_CONTAINER_PORT))
  except:
    raise ValueError("Network port should be in the range of [{}, {}]".format(constant.DEV_PROXY_MIN_CONTAINER_PORT,constant.DEV_PROXY_MAX_CONTAINER_PORT))
  return p

def check_network_whitelist(white):
  whitelist = white.strip().split(" ")
  for i in whitelist:
    if not check_cidr(i):
      raise ValueError("Network whitelist should be in CIDR format")
  return whitelist

def check_cidr(cidr):
  try:
    if cidr == None or not isinstance(cidr, str) or cidr.count(".") != 3:
      return False
    netaddr.IPNetwork(cidr)
    return True
  except Exception as e:
    return False